#tasks {
    width: 100%;
}

.QuickFinderNoMatch {
    display: none;
}
#tasks_empty {
    padding: 4px;
    font-style: italic;
}

#quicksearch {
    display: inline;
}
#quicksearch input {
    background-image: url("graphics/search.png");
    background-repeat: no-repeat;
    background-position: 2px center;
    padding: 1px 0 1px 20px;
}
#quicksearch a {
    display: inline-block;
    padding: 2px 4px;
}

/* Redbox styles. */
#RB_window {
    top: 50px !important;
    margin-top: 0 !important;
}

.overdue, .overdue td, .overdue a, .overdue td a {
    color: #f00;
}
.closed, .closed td, .closed a, .closed td a {
    color: #aaa;
}
.closed a, .closed td a {
    text-decoration: line-through;
}

.pri-1 {
    color: #f00;
}
.pri-2 {
    color: #f30;
}
.pri-3 {
    color: #000;
}
.pri-4 {
    color: #03f;
}
.pri-5 {
    color: #00f;
}

.horde-header {
    padding: 10px;
    background-color: #eceff0;
    border-bottom: 1px solid #d0d0d0;
}
.horde-header tr {
    vertical-align: top;
}
.horde-header td {
    padding: 0 0 1px 0;
}
.horde-header td.horde-label {
    font-weight: bold;
    padding-right: 4px;
    text-align: right;
    white-space: nowrap;
}

.taskBody {
    padding: 10px;
    background: #fff;
    font-family: "Lucida Console",Courier,"Courier New";
    white-space: normal;
    overflow: auto;
}

#tasks-body tr {
    line-height: 20px;
}

#dueimg, #startimg, #recur_endimg {
    cursor: pointer;
    padding-left: 3px;
}

.nag-tags-related, .nag-tags-browsing {
    padding-bottom: 12px;
    padding-left: 5px;
}

.horde-tags li a img {
    padding-left: 5px;
}

/* Icons. */
.nag-list {
    background-image: url("graphics/nag.png");
}
.nag-search {
    background-image: url("graphics/search.png");
}
.horde-new-extra {
    background: none;
}
.horde-new-extra a {
    background: url("graphics/add.png") no-repeat;
}

.nag-complete {
    background-image: url("graphics/complete.png");
}
.nag-edit {
    background-image: url("graphics/edit.png");
}
.nag-delete {
    background-image: url("graphics/delete.png");
}
.nag-new {
    background-image: url("graphics/add.png");
}

@media print {
    .header .smallheader, .tabset, .notices, #quicksearchL, #quicksearch {
        display: none;
    }
}
